/*      $Id$
 *
 * Created by René Hexel on 18/12/09.
 *
 * Copyright (c) 2009-2012 Rene Hexel. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *       This product includes software developed by Rene Hexel.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#import <UIKit/UIKit.h>

@class PROGLevel;       // forward declaration of level for protocol
@class PROGSprite;      // forward declaration of sprite for interface

/**
 * Level Delegate: tells the level important information
 * about the drawing canvas.
 */
@protocol PROGLevelDelegate <NSObject>

- (CGRect) boundsForLevel: (PROGLevel *) level;

@end


/**
 * Game Level
 */
@interface PROGLevel: NSObject

@property (nonatomic, assign) id<PROGLevelDelegate> delegate;   /// canvas delegate

@property (nonatomic, assign) int levelNumber;                  /// current level number
@property (nonatomic, assign, getter=isLifeLost) BOOL lifeLost; /// was a life lost?
@property (nonatomic, assign, getter=isFinished) BOOL finished; /// are we done yet?
@property (nonatomic, assign) BOOL endOfLevelAnimationDone;     /// end of end of level animation?

@property (nonatomic, retain) NSMutableArray *players;          /// player object(s)
@property (nonatomic, retain) NSMutableArray *sprites;          /// foreground objects
@property (nonatomic, retain) NSMutableArray *backgroundSprites;/// background objects

@property (nonatomic, assign) long maximumVelocityForLevel;     /// maximum speed
@property (nonatomic, assign) NSInteger maxNumberOfSprites;     /// limit of sprites

+ (PROGLevel *) initialLevel;                           /// convenience method

- initWithLevelNumber: (NSInteger) currentLevelNumber;  /// what's the current level?
- (PROGLevel *) nextLevel;                              /// return the next level

- (void) setupLevel;                                    /// (re)initialise level

- (NSArray *) imageNamesForCurrentLevel;                /// images for current level

- (void) setInitialPositionForPlayer: (PROGSprite *) player;
- (void) setInitialPositionForSprite: (PROGSprite *) sprite;

- (void) activate;                                      /// make level active
- (void) deactivate;                                    /// make level inactive

- (void) addSprite;                                     /// add another sprite
- (void) moveSprites;                                   /// move all sprites

- (void) addPlayer;                                     /// add a player
- (BOOL) movePlayers;                                   /// move players, report collision

- (void) addPlayers: (NSArray *) playerArray;           /// add a list of existing players

- (void) setAccelerationX: (float) x  y: (float) y;     /// set player acceleration
- (void) setSpeedX: (float) x  y: (float) y;            /// set player speed

- (void) drawObjectsInRect: (CGRect) rect
        withCurrentContext: (CGContextRef) context;     /// draw objects

- (void) startEndOfLevelAnimation;                      /// animate level switch

@end
