/*      $Id$
 *
 *   Created by Rene Hexel on 5/01/11.
 *
 * Copyright (c) 2011 Rene Hexel. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *       This product includes software developed by Rene Hexel.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * -----------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or
 * modify it under the above terms or under the terms of the GNU
 * General Public License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see http://www.gnu.org/licenses/
 * or write to the Free Software Foundation, Inc., 51 Franklin Street,
 * Fifth Floor, Boston, MA  02110-1301, USA.
 */
#import "PROGLevel_1.h"

#pragma mark - Level 1 private properties and methods

@interface PROGLevel_1 ()

@property (assign) NSTimer *levelTimer;

@end

#pragma mark - Level 1 implementation

@implementation PROGLevel_1

@synthesize levelTimer, timeBetweenSprites;

- (void) setupLevel
{
        int speedFactor = 1;
        int spriteFactor = 1;

        [super setupLevel];

        if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)       // bigger screen?
        {
                speedFactor = 2;
                spriteFactor = 2;
        }

        self.timeBetweenSprites = LEVEL_1_SPRITE_TIME * self.levelNumber;
        self.maximumVelocityForLevel = MAX_LEVEL_1_VELOCITY * (self.levelNumber-1) * speedFactor;
        self.maxNumberOfSprites = MAX_LEVEL_1_SPRITES * spriteFactor - self.levelNumber;

        if (self.maxNumberOfSprites < 1) self.maxNumberOfSprites = 1;
}


- (NSArray *) imageNamesForCurrentLevel
{
        return [NSArray arrayWithObject: @"Platform.png"];      // just 1 image
}


- (void) activate
{
        levelTimer = [NSTimer scheduledTimerWithTimeInterval: self.timeBetweenSprites
                                                      target: self
                                                    selector: @selector(addSprite)
                                                    userInfo: self
                                                     repeats: YES];
        [self addSprite];
        [self performSelector: @selector(addPlayer)
                   withObject: self
                   afterDelay: 1.5];
}


- (void) deactivate
{
        [levelTimer invalidate];
        levelTimer = nil;
}


- (void) dealloc
{
        [self deactivate];
        [super dealloc];
}
@end
