/*      $Id: PROGSound.m 8 2009-03-29 10:58:29Z rh $
 *
 * Created by René Hexel on 29/03/09.
 *
 * Copyright (c) 2009-2012 Rene Hexel. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *       This product includes software developed by Rene Hexel.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#import "PROGSound.h"

#pragma mark - Private Interface

@interface PROGSound ()         /// category for private methods

- (void) registerCallback;      /// register completion callback with system
- initWithSoundID: (SystemSoundID) sysSoundID;  /// use given system sound ID

@end

#pragma mark - PROGSound Implementation

@implementation PROGSound

@synthesize playing;
@synthesize needsDispose;

static void soundCompletedPlaying(SystemSoundID soundID, void *object)
{
        PROGSound *self = (id) object;
        self.playing = NO;
}


- (void) registerCallback
{
        AudioServicesAddSystemSoundCompletion(soundID, NULL, NULL,
                                              soundCompletedPlaying, self);
}


- initWithContentsOfFile: (NSString *) fileName
{
        if (!(self = [super init])) return nil;

        NSURL *soundURL = [NSURL fileURLWithPath: fileName];
        if (AudioServicesCreateSystemSoundID((CFURLRef) soundURL, &soundID) !=
            kAudioServicesNoError)
        {
                [self release];         // error: release self and return nil
                return nil;
        }
        needsDispose = YES;             // need to dispose of sound ID on dealloc

        [self registerCallback];

        return self;
}


- initWithSoundID: (SystemSoundID) sysSoundID
{
        if (!(self = [super init])) return nil;

        soundID = sysSoundID;

        [self registerCallback];

        return self;
}

- (void) play
{
        if (soundID != kSystemSoundID_Vibrate)          // no callback here
                self.playing = YES;                     // the sound is playing
        AudioServicesPlaySystemSound(soundID);
}

#pragma mark - Convenience Methods

+ (PROGSound *) soundNamed: (NSString *) soundName
{
        @try
        {
                NSString *soundFile = [NSBundle pathForResource: soundName
                                                         ofType: @"aiff"
                                                    inDirectory: [[NSBundle mainBundle] resourcePath]];
                return [[[self alloc] initWithContentsOfFile: soundFile] autorelease];

        }
        @catch (NSException *exception)
        {
                NSLog(@"Could not open %@: %@", soundName, [exception reason]);
        }
        return nil;
}


+ (PROGSound *) vibrationSound
{
        return [[[self alloc] initWithSoundID: kSystemSoundID_Vibrate]
                autorelease];
}

#pragma mark - Memory Management

- (void) dealloc
{
        AudioServicesRemoveSystemSoundCompletion(soundID);
        
        if (needsDispose)
                AudioServicesDisposeSystemSoundID(soundID);     // release sound
        [super dealloc];
}

@end
