/*      $Id$
 *
 * Created by René Hexel on 23/03/08.
 *
 * Copyright (c) 2008-2012 Rene Hexel. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *       This product includes software developed by Rene Hexel.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#import <UIKit/UIKit.h>

@interface PROGSprite: NSObject

@property (nonatomic, retain) UIImage   *image;         /// image to display
@property (nonatomic, assign) CGRect     dimensions;    /// position and size
@property (nonatomic, assign) CGPoint    velocity;      /// sprite velocity
@property (nonatomic, assign) CGPoint    acceleration;  /// sprite acceleration
@property (nonatomic, assign) float      angularVelocity;       /// rotation speed
@property (nonatomic, assign) float      angle;         /// current rotation angle
@property (nonatomic, assign) float      alpha;         /// drawing opacity
@property (nonatomic, assign) float      lifeTime;      /// lifetime of sprite

@property (nonatomic, assign) CGPoint    position;      /// convenience property

- initWithImageNamed: (NSString *) file;                /// load image from given file
- initWithImage: (UIImage *) anImage;                   /// construct sprite with image

- (BOOL) intersectsWithRect: (CGRect) rect;             /// check for intersection
- (BOOL) hasCollisionWith: (PROGSprite *) otherObject;  /// got collision?
- (BOOL) handleCollisionWith: (PROGSprite *) otherObject; /// handle collision
- (void) draw;                                          /// draw at current position
- (void) drawInContext: (CGContextRef) currentContext;  /// draw in given context
- (void) animate;                                       /// single animation step

@end
